@@ -58,12 +58,14 @@ STATUS_TLS_HANDSHAKE_ERROR = 1015
 
 logger = logging.getLogger()
 
+
 class WebSocketException(Exception):
     """
     websocket exeception class.
     """
     pass
 
+
 class WebSocketConnectionClosedException(WebSocketException):
     """
     If remote host closed the connection or some network error happened,
@@ -74,6 +76,7 @@ class WebSocketConnectionClosedException(WebSocketException):
 default_timeout = None
 traceEnabled = False
 
+
 def enableTrace(tracable):
     """
     turn on/off the tracability.
@@ -87,6 +90,7 @@ def enableTrace(tracable):
             logger.addHandler(logging.StreamHandler())
         logger.setLevel(logging.DEBUG)
 
+
 def setdefaulttimeout(timeout):
     """
     Set the global timeout setting to connect.
@@ -96,12 +100,14 @@ def setdefaulttimeout(timeout):
     global default_timeout
     default_timeout = timeout
 
+
 def getdefaulttimeout():
     """
     Return the global timeout setting(second) to connect.
     """
     return default_timeout
 
+
 def _parse_url(url):
     """
     parse url and the result is tuple of
@@ -130,7 +136,7 @@ def _parse_url(url):
     elif scheme == "wss":
         is_secure = True
         if not port:
-            port  = 443
+            port = 443
     else:
         raise ValueError("scheme %s is invalid" % scheme)
 
@@ -144,6 +150,7 @@ def _parse_url(url):
 
     return (hostname, port, resource, is_secure)
 
+
 def create_connection(url, timeout=None, **options):
     """
     connect to url and return websocket object.
@@ -177,6 +184,7 @@ _MAX_CHAR_BYTE = (1<<8) -1
 # ref. Websocket gets an update, and it breaks stuff.
 # http://axod.blogspot.com/2010/06/websocket-gets-update-and-it-breaks.html
 
+
 def _create_sec_websocket_key():
     uid = uuid.uuid4()
     return base64.encodestring(uid.bytes).strip()
@@ -186,6 +194,7 @@ _HEADERS_TO_CHECK = {
     "connection": "upgrade",
     }
 
+
 class _SSLSocketWrapper(object):
     def __init__(self, sock):
         self.ssl = socket.ssl(sock)
@@ -197,6 +206,8 @@ class _SSLSocketWrapper(object):
         return self.ssl.write(payload)
 
 _BOOL_VALUES = (0, 1)
+
+
 def _is_bool(*values):
     for v in values:
         if v not in _BOOL_VALUES:
@@ -204,6 +215,7 @@ def _is_bool(*values):
 
     return True
 
+
 class ABNF(object):
     """
     ABNF frame class.
@@ -316,6 +328,7 @@ class ABNF(object):
             _d[i] ^= _m[i % 4]
         return _d.tostring()
 
+
 class WebSocket(object):
     """
     Low level WebSocket interface.
@@ -337,6 +350,7 @@ class WebSocket(object):
     get_mask_key: a callable to produce new mask keys, see the set_mask_key
       function's docstring for more details
     """
+
     def __init__(self, get_mask_key = None):
         """
         Initalize WebSocket object.
@@ -409,7 +423,11 @@ class WebSocket(object):
         else:
             hostport = "%s:%d" % (host, port)
         headers.append("Host: %s" % hostport)
-        headers.append("Origin: %s" % hostport)
+        
+        if "origin" in options:
+            headers.append("Origin: %s" % options["origin"])
+        else:
+            headers.append("Origin: %s" % hostport)
 
         key = _create_sec_websocket_key()
         headers.append("Sec-WebSocket-Key: %s" % key)
@@ -423,8 +441,8 @@ class WebSocket(object):
         header_str = "\r\n".join(headers)
         sock.send(header_str)
         if traceEnabled:
-            logger.debug( "--- request header ---")
-            logger.debug( header_str)
+            logger.debug("--- request header ---")
+            logger.debug(header_str)
             logger.debug("-----------------------")
 
         status, resp_headers = self._read_headers()
@@ -547,8 +565,7 @@ class WebSocket(object):
                 self.send_close()
                 return (frame.opcode, None)
             elif frame.opcode == ABNF.OPCODE_PING:
-                self.pong("Hi!")
-
+                self.pong(frame.data)
 
     def recv_frame(self):
         """
@@ -603,8 +620,6 @@ class WebSocket(object):
             raise ValueError("code is invalid range")
         self.send(struct.pack('!H', status) + reason, ABNF.OPCODE_CLOSE)
 
-
-
     def close(self, status = STATUS_NORMAL, reason = ""):
         """
         Close Websocket object
@@ -662,6 +677,7 @@ class WebSocket(object):
                 break
         return "".join(line)
 
+
 class WebSocketApp(object):
     """
     Higher level of APIs are provided.
@@ -698,11 +714,13 @@ class WebSocketApp(object):
         self.get_mask_key = get_mask_key
         self.sock = None
 
-    def send(self, data):
+    def send(self, data, opcode = ABNF.OPCODE_TEXT):
         """
-        send message. data must be utf-8 string or unicode.
+        send message.
+        data: message to send. If you set opcode to OPCODE_TEXT, data must be utf-8 string or unicode.
+        opcode: operation code of data. default is OPCODE_TEXT.
         """
-        if self.sock.send(data) == 0:
+        if self.sock.send(data, opcode) == 0:
             raise WebSocketConnectionClosedException()
 
     def close(self):
@@ -751,6 +769,6 @@ if __name__ == "__main__":
     ws.send("Hello, World")
     print "Sent"
     print "Receiving..."
-    result =  ws.recv()
+    result = ws.recv()
     print "Received '%s'" % result
     ws.close()
